/****************************************************************************/
/*                                                                          */
/*                   Copyright 2003 University of Glasgow                   */
/*                                                                          */
/*                           All rights reserved.                           */
/*                                                                          */
/****************************************************************************/

/****************************************************************************/
/* FILE          : Main.java                                                */
/* DESCRIPTION   : Top-level driver for the Simple Functional Language      */
/*                 (no mutually recursive functions)                        */
/*                                                                          */
/* AUTHOR        : S.J.Gay                                                  */
/* DATE          : 28th August 2003                                         */
/*                                                                          */
/* LAST MODIFIED : S.J.Gay                                                  */
/* DATE          : 8th October 2003                                         */
/****************************************************************************/

package sfl;

import sfl.node.*;
import sfl.lexer.*;
import sfl.parser.*;
import sfl.error.*;
import sfl.checker.*;
import sfl.types.*;
import java.io.*;
import java.util.*;

public class Main {

    private static Lexer scanner;
    private static Parser parser;

    /** The AST representing the source program. */
    private static Node theAST;

    static void check (String sourceName, boolean mutual) {
	try {
	    scanner = new Lexer(new PushbackReader(
		      new BufferedReader(new FileReader(sourceName)), 1024));
	} catch(FileNotFoundException e)
	    {System.out.println("Can't access source file " + sourceName);
	    System.exit(1);};

	parser = new Parser(scanner);
        try {
	    theAST = parser.parse();
	} catch(Exception e)
	    {System.out.println(e);
	     System.exit(1);}

	if (mutual) {
	    Mutual m = new Mutual();
	    theAST.apply(m);
	    ErrorTable errorTable = m.getErrorTable();
	    if (errorTable.hasErrors()) {
		errorTable.output(System.out,
			     "while processing " + sourceName);
		System.exit(1);
	    };
	    Checker chk = new Checker(); 
	    chk.allowMutualRecursion();
	    chk.setEnv(m.getEnv());
	    theAST.apply(chk);
	    errorTable = chk.getErrorTable();
	    if (errorTable.hasErrors()) {
		errorTable.output(System.out,
			     "while processing " + sourceName);
		System.exit(1);
	    }
	    else {
		System.out.println("Expression has type " +
			       ((Type)chk.getOut(theAST)).name());
	    }
	}
	else {
	    Checker chk = new Checker(); 
	    theAST.apply(chk);
	    ErrorTable errorTable = chk.getErrorTable();
	    if (errorTable.hasErrors()) {
		errorTable.output(System.out,
			     "while processing " + sourceName);
		System.exit(1);
	    }
	    else {
		System.out.println("Expression has type " +
			       ((Type)chk.getOut(theAST)).name());
	    }
	}
    }

    public static void main(String[] args) {
	String sourceName;
	boolean mutual;
        if (args.length < 1) {
            System.out.println("Usage: check [-mutual] filename");
            System.exit(1);
        }
	if (args[0].equals("-mutual")) {
	    mutual = true;
	    sourceName = args[1];
	}
	else {
	    mutual = false;
	    sourceName = args[0];
	};
        check(sourceName,mutual);
    }
}
