/****************************************************************************/
/*                                                                          */
/*                   Copyright 2003 University of Glasgow                   */
/*                                                                          */
/*                           All rights reserved.                           */
/*                                                                          */
/****************************************************************************/

/****************************************************************************/
/* FILE          : Searchtable.java                                         */
/* DESCRIPTION   : Like a hash table indexed by strings, but using string   */
/*                 comparison                                               */
/*                                                                          */
/* AUTHOR        : S.J.Gay                                                  */
/* DATE          : 18th August 2003                                         */
/*                                                                          */
/* LAST MODIFIED : S.J.Gay                                                  */
/* DATE          : 18th September 2003                                      */
/****************************************************************************/

package sfl.env;

import java.util.*;

/* A search table is a pair of vectors, one containing strings (indices) and
   the other containing stored objects.

  The operations of the Dictionary interface are implemented, with more
  specific types. */

public class Searchtable {
    Vector keys;
    Vector values;

    public Searchtable() {
	keys = new Vector(1,1);
	values = new Vector(1,1);
    }

    public Enumeration elements() {
	return values.elements();
    }

    public Enumeration keys() {
	return keys.elements();
    }

    public boolean isEmpty() {
	return keys.isEmpty();
    }

    int indexOf(String key) {
	int size = keys.size();
	for (int i = 0; i < size; i++) {
	    if (((String)keys.elementAt(i)).equals(key)) {
		return i;
	    }
	};
	return -1;
    }

    public Object put(String key, Object value) {
	int index = indexOf(key);
	if (index == -1) {
	    keys.addElement(key);
	    values.addElement(value);
	    return null;
	}
	else {
	    return values.elementAt(index);
	}
    }

    public Object get(String key) {
	int index = indexOf(key);
	if (index == -1) {
	    return null;
	}
	else {
	    return values.elementAt(index);
	}
    }

    public int size() {
	return keys.size();
    }

    public Object remove(String key) {
	int index = indexOf(key);
	if (index == -1) {
	    return null;
	}
	else {
	    keys.removeElementAt(index);
	    Object value = values.elementAt(index);
	    values.removeElementAt(index);
	    return value;
	}
    }
}
